/*
 * linkloop: Loopback testing for layer-2 connectivity
 *           via 802.2LLC TEST message.
 *
 * Tested against HPUX-11.0
 *
 * Written by Oron Peled <oron@actcom.co.il>
 *
 * Some code snippets adapted from spak
 * (http://www.xenos.net/software/spak/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

static char rcsid[] = "$id:$";

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>	/* for IF_NAMESIZE, IFHWADDRLEN */
#include <arpa/inet.h>
#include "config.h"
#include "linkloop.h"

#define	TEST_CMD	0xE3	/* From 802.2LLC (HPUX sends 0xF3) */

char	*program = NULL;

/* Sometime we'll make them configurable */
int	timeout = 1;
int	retries = 3;
int	debug = 0;
char	*iface = "eth0";

void sig_alarm(int signo) {
	fprintf(stderr, "TIMEOUT\n");
}

void set_timeout() {
	struct sigaction sa;

	sa.sa_handler = sig_alarm;
	sa.sa_flags = 0;
	sigfillset(&sa.sa_mask);	/* Block spurious signals during handler */
	sigdelset(&sa.sa_mask, SIGINT);	/* But normal signals should kill us */
	sigdelset(&sa.sa_mask, SIGQUIT);
	sigdelset(&sa.sa_mask, SIGTERM);
	if(sigaction(SIGALRM, &sa, NULL) < 0) {
		perror("sigaction");
		exit(1);
	}
	if(alarm(timeout) < 0) {
		perror("alarm");
		exit(1);
	}
}

int main(int argc, char *argv[]) {
	int sock;
	int i;
	u_int8_t mac_src[IFHWADDRLEN];
	u_int8_t mac_dst[IFHWADDRLEN];
	struct llc_packet spack;
	struct llc_packet rpack;

	program = argv[0];

	if(argc <= 1) {
		fprintf(stderr, "Usage: %s <target_mac>\n", program);
		return 1;
	}
	if(!parse_address(mac_dst, argv[1])) {
		fprintf(stderr, "%s: bad DST address - %s\n", program, argv[1]);
		exit(1);
	}

	/* Open a socket */
	if((sock = socket(AF_INET, SOCK_PACKET, htons(ETH_P_802_2))) == -1) {
		perror("socket");
		return 1;
	}
	get_hwaddr(sock, iface, mac_src);
	printf("Testing via %s (HW addr %s)\n", iface, mac2str(mac_src));

	mk_test_packet(&spack, mac_src, mac_dst, 0);
	if(debug)
		dump_packet(&spack);

	for(i = 0; i < retries; i++) {
		set_timeout();
		send_packet(sock, iface, &spack);
		if(recv_packet(sock, &rpack))
			break;
		printf("Retry...\n");
	}
	if(i >= retries)
		printf("-- NO RESPONSE --\n");
	else if(memcmp(spack.eth_hdr.ether_dhost, rpack.eth_hdr.ether_shost, IFHWADDRLEN) == 0)
		printf("-- OK --\n");
	else {
		printf("-- BAD RESPONSE --\n");
		dump_packet(&rpack);
	}
	return 0;
}
